*************************************************************************************
*														*
*	A starfiled simulation. Double buffering is used to render a swooping and	*
*	spinning starfield.	2008/07/30								*
*														*
*	While this all looks nice there is a shortcomming, no depth sorting is done	*
*	so nearer stars may be obscured by more distant stars depending on the		*
*	order in which they are drawn.								*
*														*
*	Use [S] to toggle the status display on/off and the following keys ..		*
*														*
*		[x] plus [curup] to increase the x spin						*
*		[y] plus [curup] to increase the y spin						*
*		[z] plus [curup] to increase the z spin						*
*														*
*		[x] plus [curdn] to decrease the x spin						*
*		[y] plus [curdn] to decrease the y spin						*
*		[z] plus [curdn] to decrease the z spin						*
*														*
*		[currt] to increase the velocity							*
*		[curlt] to decrease the velocity							*
*														*
*	The F2, F3 and F4	keys can be used to select a screen size of 640 x 480,	*
*	800 x 600 and 1024 x 768 respectively.							*
*														*
*	This version for Sim68K 3.6 or later							*
*														*
*	More 68000 and other projects can be found on my website at ..			*
*														*
*	 http://members.lycos.co.uk/leeedavison/index.html					*
*														*
*	mail : leeedavison@lycos.co.uk								*
*														*
*************************************************************************************
*
* constants

star_count		EQU	400			* number of stars

spin_max		EQU	17			* maximum spin
spin_min		EQU	0			* minimum spin
spin_stp		EQU	1			* spin step size

velocity_max	EQU	21			* maximum velocity
velocity_min	EQU	0			* minimum velocity
velocity_stp	EQU	1			* velocity step size

init_spin_x		EQU	2			* initial x spin
init_spin_y		EQU	3			* initial x spin
init_spin_z		EQU	5			* initial x spin
init_vel		EQU	7			* initial velocity

box_size		EQU	$100			* environment box +/- limits

view_off		EQU	450			* view y offset

*************************************************************************************
*
* code start

	ORG	$1000

start
	MOVEQ		#10,d1			* OR mode drawing
	MOVEQ		#92,d0			* set draw mode
	TRAP		#15

	MOVEQ		#17,d1			* enable double buffering
	MOVEQ		#92,d0			* set draw mode
	TRAP		#15

	MOVE.w	#$FF00,d1			* clear screen
	MOVEQ		#11,d0			* position cursor
	TRAP		#15

	BSR		Initialise			* go setup everything

* main loop. everything is done in this loop, the overall speed of the loop is set by
* waiting for a fixed time to pass at the end of the loop

loop
	MOVEQ		#8,d0				* get time in 1/100 ths seconds
	TRAP		#15

	MOVE.l	d1,-(sp)			* push time on the stack

	MOVEQ		#94,d0			* copy screen buffer to main
	TRAP		#15

	BSR		screen_size			* test and handle widow size changes
	BSR		do_keys			* test and handle parameter keypresses
	BSR		keys_ltrt			* test the [crlt][crrt] keys

	BSR		add_spins			* add spins to current rotation angles
	BSR		move_stars			* move the stars
	BSR		translate			* translate all points

	MOVE.w	#$FF00,d1			* clear the screen
	MOVEQ		#11,d0			* position cursor
	TRAP		#15

	BSR		draw_status			* draw the status
	BSR		draw_stars			* draw the stars

* now wait for a while. this limits the loop to ten redraws a second without which
* this would run way to fast if the machine is capable

	MOVE.l	(sp)+,d7			* restore time in 1/100 ths from stack
wait_100ms
	MOVEQ		#8,d0				* get time in 1/100 ths seconds
	TRAP		#15

	SUB.l		d7,d1				* subtract previous time from current time
	CMP.b		#$09,d1			* compare with 9/100ths
	BMI.s		wait_100ms			* loop if time not up yet

* if you want to see how fast it can go comment out the BMI.s above

	BRA		loop				* loop forever


*************************************************************************************
*
* check the [F2], [F3] and [F4] keys. set the screen size to 640 x 480, 800 x 600 or
* 1024 x 768 if the corresponding key has been pressed

screen_size
	MOVE.l	#$71007273,d1		* [F2], [], [F3] and [F4] keys
	MOVEQ		#19,d0			* check for keypress
	TRAP		#15

	MOVE.l	d1,d2				* copy result
	BEQ.s		notscreen			* skip screen size if no F key

	MOVE.l	#$028001E0,d1		* set 640 x 480
	TST.l		d2				* test result
	BMI		setscreen			* if F2 go set window size

	MOVE.l	#$03200258,d1		* set 800 x 600
	TST.w		d2				* test result
	BMI		setscreen			* if F3 go set window size

							* else was F4 so ..
	MOVE.l	#$04000300,d1		* set 1024 x 768
setscreen
	CMP.l		screen_width(a3),d1	* compare with current screen size
	BEQ.s		notscreen			* if already set skip setting it now

	MOVE.l	d1,screen_width(a3)	* save the window size
	MOVEQ 	#33,d0			* get/set window size
	TRAP		#15

notscreen
	RTS


*************************************************************************************
*
* test and handle parameter keypresses
*
* [S] toggles the status display on/off
*
* [x] plus [curup] to increase the x spin
* [y] plus [curup] to increase the y spin
* [z] plus [curup] to increase the z spin
*
* [x] plus [curdn] to decrease the x spin
* [y] plus [curdn] to decrease the y spin
* [z] plus [curdn] to decrease the z spin

do_keys
	MOVEQ		#$53,d1			* [][][][S] key
	MOVEQ		#0,d2				* clear longword
	MOVE.w	key_status(a3),d2		* get the last key status
	BSR		test_keys			* go test key
	BEQ		no_toggle_stat		* if no key press skip status toggle

	EOR.w		#-1,status(a3)		* toggle the status flag
no_toggle_stat
	MOVE.w	d2,key_status(a3)		* save the last key syatus

	MOVEQ		#'X',d1			* [][][][X] key
	MOVEQ		#19,d0			* check for keypress
	TRAP		#15

	TST.b		d1				* test the result and clear last key
	BEQ.s		not_x				* if not x go try y

	MOVEQ		#0,d2				* clear longword
	MOVE.w	key_xupdn(a3),d2		* get the last key status

	MOVE.w	spin_x(a3),d6		* get x spin
	BSR		test_updn			* test the [crup][crdn] keys
	MOVE.w	d6,spin_x(a3)		* save x spin
not_x
	MOVE.w	d2,key_xupdn(a3)		* save the last key status

	MOVEQ		#'Y',d1			* [][][][Y] key
	MOVEQ		#19,d0			* check for keypress
	TRAP		#15

	TST.b		d1				* test the result
	BEQ.s		not_y				* if not y go try z

	MOVEQ		#0,d2				* clear longword
	MOVE.w	key_yupdn(a3),d2		* get the last key status

	MOVE.w	spin_y(a3),d6		* get y spin
	BSR		test_updn			* test the [crup][crdn] keys
	MOVE.w	d6,spin_y(a3)		* save y spin
not_y
	MOVE.w	d2,key_yupdn(a3)		* save the last key syatus

	MOVEQ		#'Z',d1			* [][][][Z] key
	MOVEQ		#19,d0			* check for keypress
	TRAP		#15

	TST.b		d1				* test the result
	BEQ.s		not_z				* if not z just exit

	MOVEQ		#0,d2				* clear longword
	MOVE.w	key_zupdn(a3),d2		* get the last key status

	MOVE.w	spin_z(a3),d6		* get z spin
	BSR		test_updn			* test the [crup][crdn] keys
	MOVE.w	d6,spin_z(a3)		* save z spin
not_z
	MOVE.w	d2,key_zupdn(a3)		* save the last key syatus
	RTS


*************************************************************************************
*
* test the [crup][crdn] keys. increase or decrease the spin, check the spin is in
* range and return the in range value. enter with the spin in d6.w, returns modified
* spin in d6.w

test_updn
	MOVEQ		#0,d4				* clear delta spin

	MOVE.l	#$00002628,d1		* [][][crup][crdn] keys
	BSR		test_keys			* go test key

	TST.w		d1				* test [crup] key pressed
	BPL		no_crup			* if not [crup] skip delta set

	MOVE.w	#spin_stp,d4		* set +ve delta spin
no_crup
	TST.b		d1				* test [crdn] key pressed
	BPL		no_crdn			* if not [crdn] skip delta set

	MOVE.w	#-spin_stp,d4		* set -ve delta spin
no_crdn
	ADD.w		d6,d4				* make new spin

	CMP.w		#spin_max,d4		* compare new spin with max
	BGT.s		spin_bad			* if > max skip spin set

	CMP.w		#spin_min,d4		* compare new spin with min
	BLT.s		spin_bad			* if < min skip spin set

	MOVE.w	d4,d6				* copy new spin
spin_bad
	RTS


*************************************************************************************
*
* test the [crlt][crrt] keys. increase or decrease the velocity within range.

keys_ltrt
	MOVEQ		#0,d2				* clear longword
	MOVE.w	key_ltrt(a3),d2		* get the last key status

	MOVE.l	#$00002527,d1		* [][][crlt][crrt] keys
	BSR		test_keys			* go test key

	MOVE.w	d2,key_ltrt(a3)		* save the last key syatus
	MOVEQ		#0,d3				* clear delta velocity

	TST.w		d1				* test [crlt] key pressed
	BPL		no_crlt			* if not [crlt] skip delta set

	MOVEQ		#-velocity_stp,d3		* set -ve delta velocity
no_crlt
	TST.b		d1				* test [crrt] key pressed
	BPL		no_crrt			* if not [crrt] skip delta set

	MOVEQ		#velocity_stp,d3		* set +ve delta velocity
no_crrt
	ADD.w		delta_y(a3),d3		* make new velocity

	CMP.w		#velocity_max,d3		* compare new velocity with max
	BGT.s		velocity_bad		* if > max skip velocity set

	CMP.w		#velocity_min,d3		* compare new velocity with min
	BLT.s		velocity_bad		* if < min skip velocity set

	MOVE.w	d3,delta_y(a3)		* save new velocity
velocity_bad
	RTS


*************************************************************************************
*
* test key(s) d1.l, last state in d2.l. returns new pressed in d1.l and new state in
* d2.l

test_keys
	MOVE.l	d0,-(sp)			* save register
	MOVEQ		#19,d0			* check for keypress
	TRAP		#15

	MOVE.l	d1,d0				* copy the result, each byte is $00 if key
							* up, $FF if key down
	EOR.l		d2,d1				* compare with last key(s) state, each byte
							* is now $FF if a key has changed or $FF if
							* a key has not changed
	MOVE.l	d0,d2				* save this result as last
	MOVE.l	(sp)+,d0			* restore register
	AND.l		d2,d1				* make each byte $FF if key just pressed or
							* $00 if key not just pressed, set Zb status
	RTS


*************************************************************************************
*
* draw the current settings

draw_status
	TST.w		status(a3)			* test the status flag
	BEQ		exit_status			* if not on just exit

	MOVE.w	#$0201,d1			* set 2,1
	MOVEQ		#11,d0			* position cursor
	TRAP		#15

	LEA		string(pc),a1		* point to string
	MOVEQ		#14,d0			* display the NULL terminated string at (a1)
	TRAP		#15

	MOVE.w	#$0E01,d1			* set 14,1
	MOVE.w	spin_x(a3),d2		* x spin
	BSR		display_number		* display d1.l in base 10

	MOVE.w	#$0E02,d1			* set 14,2
	MOVE.w	spin_y(a3),d2		* y spin
	BSR		display_number		* display d1.l in base 10

	MOVE.w	#$0E03,d1			* set 14,3
	MOVE.w	spin_z(a3),d2		* z spin
	BSR		display_number		* display d1.l in base 10

	MOVE.w	#$0E04,d1			* set 14,5
	MOVE.w	delta_y(a3),d2		* y velocity
	BSR		display_number		* display d1.l in base 10

exit_status
	RTS

string
	dc.b	'X spin    :     [x][curdn]- [x][curup]+',$0D,$0A
	dc.b	'  Y spin    :     [y][curdn]- [y][curup]+',$0D,$0A
	dc.b	'  Z spin    :     [z][curdn]- [z][curup]+',$0D,$0A
	dc.b	'  Velocity  :        [currt]-    [curlt]+'
	dc.b	0
	ds.w	0					* ensure even


*************************************************************************************
*
* output number, cursor position in d1.w, number in d2.w

display_number
	CMP.w		#10,d2			* compare with 10
	BCC		print_number		* if >= 10 go output the number

	ADD.w		#$0100,d1			* else increment the x position
print_number
	MOVEQ		#11,d0			* position cursor
	TRAP		#15

	MOVEQ		#0,d1				* clear longword
	MOVE.w	d2,d1				* copy the number to display
	MOVEQ		#$0A,d2			* set base 10
	MOVEQ		#15,d0			* display d1.l in base d2.b
	TRAP		#15

	RTS


*************************************************************************************
*
* convert co-ordinate sets. first the object co-ordinates x,y,z, held in d4,d5,d6,
* are converted to world co-ordinates by multiplying x,y,z by the object yaw and then
* adding the object's world co-ordinate origin.
*
* returns xa,ya in d0,d1 and wx,wy,wz in d4,d5,d6

* wx& = (x& * cosa) - (y& * sina)
* wy& = (x& * sina) + (y& * cosa) + view_off&
* wz& = z&

make_xy
	ADD.w		#view_off,d5		* wy = y + view_off

* now the world object co-ordinates are transformed to the screen co-ordinates

	MOVE.w	d4,d0				* copy x co-ordinate
	MOVE.w	d5,d1				* copy y co-ordinate
	BNE.s		not_zero			* skip fudge if not zero

	MOVEQ		#1,d1				* ensure no divide by zero error
not_zero

* xa& = x centre - x size * wx& / wy&
* ya& = y centre - x size * wz& / wy&

	MULS.w	screen_width(a3),d0	* multiply and make longword
	NEG.l		d0				* change sign for - field of view
	DIVS.w	d1,d0				* divide by distance
	MOVE.w	screen_width(a3),d2	* get the screen width
	ASR.w		#1,d2				* / 2 to get x centre
	ADD.w		d2,d0				* d0 is now screen x

	MOVE.w	d6,d2				* copy z co-ordinate

	MULS.w	screen_width(a3),d2	* multiply and make longword
	NEG.l		d2				* change sign for - field of view
	DIVS.w	d1,d2				* divide by distance
	MOVE.w	screen_height(a3),d1	* get the screen height
	ASR.w		#1,d1				* / 2 to get y centre
	ADD.w		d2,d1				* d1 is now screen y

	RTS


*************************************************************************************
*
* draw the stars. the brightness of each star is set to one of three values depending
* on the stars distance from the viewpoint

draw_stars
	MOVE.l	#$00FFFFFF,d1		* set the brightest colour
	MOVEQ		#80,d0			* set pen colour
	TRAP		#15

	LEA		x_buffer(a3),a0		* point to the x value buffer
	LEA		y_buffer(a3),a1		* point to the y value buffer
	LEA		z_buffer(a3),a2		* point to the z value buffer

	MOVE.w	#star_count-1,d7		* get the number of stars to draw
draw_stars_loop
	MOVE.w	(a0)+,d4			* set point x
	MOVE.w	(a1)+,d5			* set point y
	MOVE.w	(a2)+,d6			* set point z

	BSR		make_xy			* convert to world x,y,z and screen x,y
	MOVEM.l	d0-d1,-(sp)			* save x,y

							* now set the brightness for the star
	MOVE.w	d5,d0				* copy the world y
	SUB.w		#view_off,d0		* subtract the viewpoint offset

	MOVE.l	#$003F3F3F,d1		* set the dimmest colour
	CMPI.w	#$80,d0			* compare the star y with $80
	BGE.s		set_bright			* if futher go set the brightness

	MOVE.l	#$007F7F7F,d1		* set the mid colour
	CMPI.w	#-$80,d0			* compare the star y with -$80
	BGE.s		set_bright			* if futher go set the brightness

	MOVE.l	#$00FFFFFF,d1		* set the brightest colour
set_bright
	MOVEQ		#80,d0			* set pen colour
	TRAP		#15

	MOVEM.l	(sp)+,d1-d2			* restore x,y

	MOVEQ		#1,d3				* x2 = 1
	MOVEQ		#1,d4				* y2 = 1
	ADD.w		d1,d3				* z2 = x1 + 1
	ADD.w		d2,d4				* y2 = y1 + 1

	MOVEQ		#90,d0			* draw rectangle
	TRAP		#15

	DBF		d7,draw_stars_loop	* decrement and loop if more to do

	RTS


*************************************************************************************
*
* move the stars

move_stars
	LEA		y_coords(a3),a1		* point to the y co-ordinates table

	MOVE.w	delta_y(a3),d1		* get stars delta Y
	NEG.w		d1				* change delta direction

	MOVE.w	#2*box_size,d4		* wrap round box value
	MOVE.w	#star_count-1,d7		* get the number of stars to move
move_stars_loop
	MOVE.w	(a1),d3			* get star y co-ordinate
	ADD.w		d1,d3				* add stars delta y
	CMP.w		#box_size,d3		* compare with maximum y dimension
	BGT.s		y_too_big			* if y is too big go wrap it

	CMP.w		#-box_size,d3		* compare with minimum y dimension
	BGE.s		y_is_ok			* if y is ok go save it

	ADD.w		d4,d3				* wrap too large value into range
	BRA.s		y_is_ok			* go save the moved y co-ordinate

y_too_big
	SUB.w		d4,d3				* wrap too large value into range
y_is_ok
	MOVE.w	d3,(a1)+			* save moved y co-ordinate

	DBF		d7,move_stars_loop	* decrement count and loop if more to do

	RTS


*************************************************************************************
*
* translate the stars, rotating about z, y and x axes. nothing clever to see here,
* each axis is rotated individually.

translate
	MOVE.w	theta_x(a3),d0		* get the x angle
	BSR		sincos			* get SIN and COS for the x angle
	MOVE.w	d0,sin_x(a3)		* save the x angle SIN
	MOVE.w	d1,cos_x(a3)		* save the x angle COS

	MOVE.w	theta_y(a3),d0		* get the y angle
	BSR		sincos			* get SIN and COS for the y angle
	MOVE.w	d0,sin_y(a3)		* save the y angle SIN
	MOVE.w	d1,cos_y(a3)		* save the y angle COS

	MOVE.w	theta_z(a3),d0		* get the z angle
	BSR		sincos			* get SIN and COS for the z angle
	MOVE.w	d0,sin_z(a3)		* save the z angle SIN
	MOVE.w	d1,cos_z(a3)		* save the z angle COS

	LEA		x_buffer(a3),a4		* point to the translated x co-ords buffer
	LEA		y_buffer(a3),a5		* point to the translated y co-ords buffer
	LEA		z_buffer(a3),a6		* point to the translated z co-ords buffer

	LEA		x_coords(a3),a0		* point to the x co-ordinates table
	LEA		y_coords(a3),a1		* point to the y co-ordinates table
	LEA		z_coords(a3),a2		* point to the z co-ordinates table

	MOVE.w	#star_count-1,d7		* get the number of stars to translate
next_star
	MOVE.w	(a0),d4			* get the star x co-ordinate
	MULS.w	cos_z(a3),d4		* * COS(z)
	MOVE.w	(a1),d1			* get the star y co-ordinate
	MULS.w	sin_z(a3),d1		* * SIN(z)
	SUB.l		d1,d4				* x' = x COS(z) - y SIN(z)
	ADD.l		d4,d4				* adjust for scale
	SWAP		d4				* discard the fractional part

	MOVE.w	(a0)+,d5			* get the star x co-ordinate
	MULS.w	sin_z(a3),d5		* * SIN(z)
	MOVE.w	(a1)+,d1			* get the star y co-ordinate
	MULS.w	cos_z(a3),d1		* * COS(z)
	ADD.l		d1,d5				* y' = x SIN(z) + y COS(z)
	ADD.l		d5,d5				* adjust for scale
	SWAP		d5				* discard the fractional part
							* rotation about the z axis complete

	MOVE.w	(a2),d6			* get the star z co-ordinate
	MULS.w	cos_y(a3),d6		* * COS(y)
	MOVE.w	d4,d1				* copy X'
	MULS.w	sin_y(a3),d1		* * SIN(y)
	SUB.l		d1,d6				* z' = z COS(y) - x' SIN(y)
	ADD.l		d6,d6				* adjust for scale
	SWAP		d6				* discard the fractional part

	MOVE.w	(a2)+,d1			* get the star z co-ordinate
	MULS.w	sin_y(a3),d1		* * SIN(y)
	MULS.w	cos_y(a3),d4		* * COS(y)
	ADD.l		d1,d4				* x" = x' COS(y) + z SIN(y)
	ADD.l		d4,d4				* adjust for scale
	SWAP		d4				* discard the fractional part
							* rotation about the y axis complete

	MOVE.w	d5,d0				* copy y'
	MULS.w	cos_x(a3),d5		* * COS(x)
	MOVE.w	d6,d1				* copy z'
	MULS.w	sin_x(a3),d1		* * SIN(x)
	SUB.l		d1,d5				* y" = y' COS(x) - z' SIN(x)
	ADD.l		d5,d5				* adjust for scale
	SWAP		d5				* discard the fractional part

	MULS.w	sin_x(a3),d0		* y' * SIN(x)
	MULS.w	cos_x(a3),d6		* z' * COS(x)
	ADD.l		d0,d6				* z" = y' SIN(x) + z' COS(x)
	ADD.l		d6,d6				* adjust for scale
	SWAP		d6				* discard the fractional part
							* rotation about the x axis complete

	MOVE.w	d4,(a4)+			* save translated x co-ordinate
	MOVE.w	d5,(a5)+			* save translated y co-ordinate
	MOVE.w	d6,(a6)+			* save translated z co-ordinate

	DBF		d7,next_star

	RTS


*************************************************************************************
*
* add spins to current rotation angles

add_spins
	MOVE.w	spin_x(a3),d0		* get the star's x spin step
	ADD.w		d0,theta_x(a3)		* add to the star's x rotation
	MOVE.w	spin_y(a3),d0		* get the star's y spin step
	ADD.w		d0,theta_y(a3)		* add to the star's y rotation
	MOVE.w	spin_z(a3),d0		* get the star's z spin step
	ADD.w		d0,theta_z(a3)		* add to the star's z rotation
	RTS


*************************************************************************************
*
* setup stuff.

Initialise
	LEA		variables(pc),a3		* variables base address

	MOVEQ		#8,d0				* get time in 1/100 ths seconds
	TRAP		#15

	EORI.l	#$DEADBEEF,d1		* EOR with the initial PRNG seed, this must
							* result in any value but zero
	MOVE.l	d1,PRNlword(a3)		* save the initial PRNG seed

	MOVEQ		#0,d0				* clear longword
	MOVE.w	d0,status(a3)		* clear the show status flag
	MOVE.w	d0,key_status(a3)		* clear the status key last state
	MOVE.l	d0,key_xupdn(a3)		* clear the x key last state
	MOVE.l	d0,key_yupdn(a3)		* clear the y key last state
	MOVE.l	d0,key_zupdn(a3)		* clear the z key last state

	MOVE.w	#init_spin_x,spin_x(a3)	* clear spin x	
	MOVE.w	#init_spin_y,spin_y(a3)	* clear spin y	
	MOVE.w	#init_spin_z,spin_z(a3)	* clear spin z

	MOVE.w	#init_vel,delta_y(a3)	* clear delta y	

	MOVE.l	#$028001E0,d1		* set default window size to 640 x 480
	MOVE.l	d1,screen_width(a3)	* save the window size
	MOVEQ 	#33,d0			* get/set window size
	TRAP		#15

*************************************************************************************
*
* generate a random sky of stars. the pattern will be different each time because the
* initial seed is modified by the PC time at startup

make_stars
	LEA		x_coords(a3),a0		* point to the co-ordinates tables
	MOVE.w	#box_size,d2		* set the value limit
	MOVE.w	#star_count*3-1,d7	* get the number of star co-ordinates
make_stars_loop
	BSR.s		rand_pm_d2			* generate a random number in the range
							* +/-(d2.w - 1), returned in d0.w
	MOVE.w	d0,(a0)+			* save the random co-ordinate

	DBF		d7,make_stars_loop	* decrement count and loop if more to do

	RTS


*************************************************************************************
*
* generate a random value in d0.w in range -(d2.w - 1) to +(d2.w - 1)

rand_pm_d2
	BSR.s		random_n			* generate a random number
	MOVE.w	PRNlword(a3),d0		* get a random word
	EXT.l		d0				* make into longword. doing this instead of
							* just getting a longword ensures that the
							* following DIVS never overflows
	DIVS.w	d2,d0				* divide by range
	SWAP		d0				* use the signed remainder
	RTS


*************************************************************************************
*
* This is the code that generates the pseudo random sequence. A seed word located in
* PRNlword(a3) is loaded into a register before being operated on to generate the
* next number in the sequence. This number is then saved as the seed for the next
* time it's called.
*
* This code is adapted from the 32 bit version of RND(n) used in EhBASIC68. Taking
* the 19th next number is slower but helps to hide the shift and add nature of this
* generator as can be seen from analysing the output.

random_n
	MOVEM.l	d0-d2,-(sp)			* save d0, d1 and d2
	MOVE.l	PRNlword(a3),d0		* get current seed longword
	MOVEQ		#$AF-$100,d1		* set EOR value
	MOVEQ		#18,d2			* do this 19 times
Ninc0
	ADD.l		d0,d0				* shift left 1 bit
	BCC.s		Ninc1				* if bit not set skip feedback

	EOR.b		d1,d0				* do Galois LFSR feedback
Ninc1
	DBF		d2,Ninc0			* loop

	MOVE.l	d0,PRNlword(a3)		* save back to seed longword
	MOVEM.l	(sp)+,d0-d2			* restore d0, d1 and d2

	RTS


*************************************************************************************
*
* get sin(d0) in d0, cos(d0) in d1

sincos
	MOVE.w	d0,d1				* copy the angle
	ADD.w		#$80,d0			* add 1/4 rotation
	BSR.s		sin_d0			* get COS(d0) in d0
	EXG		d0,d1				* swap it with d1

*************************************************************************************
*
* get SIN(d0) in d0. d0 is a nine bit value representing a full circle with the value
* increasing as you turn widdershins

sin_d0
	BTST		#8,d0				* test angle sign
	BEQ.s		cossin_d0			* just get SIN/COS and return if +ve

	BSR.s		cossin_d0			* else get SIN/COS
	NEG.w		d0				* now do twos complement
	RTS

* get d0 from SIN/COS table

cossin_d0
	TST.b		d0				* test for >= $80
	BPL.s		a_was_less			* branch if < 1/4 circle

	NEG.b		d0				* wrap $81 to $FF to $7F to $01
a_was_less
	AND.w		#$FF,d0			* ensure word high byte clear
	ADD.w		d0,d0				* * 2 bytes per word value
	LEA		sin_cos(pc),a0		* get table base address
	MOVE.w	(a0,d0.w),d0		* get SIN/COS value
	RTS

*************************************************************************************
*
* SIN/COS table, returns values between $0000 and $7FFF. the last value should be
* $8000 but that can cause an overflow in the word length calculations and it's
* easier to fudge the table a bit. no one will ever notice.

sin_cos
	dc.w	$0000,$0192,$0324,$04B6,$0648,$07D9,$096B,$0AFB
	dc.w	$0C8C,$0E1C,$0FAB,$113A,$12C8,$1455,$15E2,$176E
	dc.w	$18F9,$1A83,$1C0C,$1D93,$1F1A,$209F,$2224,$23A7
	dc.w	$2528,$26A8,$2827,$29A4,$2B1F,$2C99,$2E11,$2F87
	dc.w	$30FC,$326E,$33DF,$354E,$36BA,$3825,$398D,$3AF3
	dc.w	$3C57,$3DB8,$3F17,$4074,$41CE,$4326,$447B,$45CD
	dc.w	$471D,$486A,$49B4,$4AFB,$4C40,$4D81,$4EC0,$4FFB
	dc.w	$5134,$5269,$539B,$54CA,$55F6,$571E,$5843,$5964
	dc.w	$5A82,$5B9D,$5CB4,$5DC8,$5ED7,$5FE4,$60EC,$61F1
	dc.w	$62F2,$63EF,$64E9,$65DE,$66CF,$67BD,$68A7,$698C
	dc.w	$6A6E,$6B4B,$6C24,$6CF9,$6DCA,$6E97,$6F5F,$7023
	dc.w	$70E3,$719E,$7255,$7308,$73B6,$7460,$7505,$75A6
	dc.w	$7642,$76D9,$776C,$77FB,$7885,$790A,$798A,$7A06
	dc.w	$7A7D,$7AEF,$7B5D,$7BC6,$7C2A,$7C89,$7CE4,$7D3A
	dc.w	$7D8A,$7DD6,$7E1E,$7E60,$7E9D,$7ED6,$7F0A,$7F38
	dc.w	$7F62,$7F87,$7FA7,$7FC2,$7FD9,$7FEA,$7FF6,$7FFE
	dc.w	$7FFF


*************************************************************************************
*
* variables used

variables						* points to variables base

	OFFSET	0				* going to use relative addressing

PRNlword	ds.l	1				* PRNG seed long word

status
	ds.w	1					* show status flag
key_status
	ds.w	1					* status key last state
key_xupdn
	ds.w	1					* x key last state
key_yupdn
	ds.w	1					* y key last state
key_zupdn
	ds.w	1					* z key last state
key_ltrt
	ds.w	1					* velocity key last state

screen_width
	ds.w	1					* screen width
screen_height
	ds.w	1					* screen height

spin_x
	ds.w	1					* stars x spin velocity
spin_y
	ds.w	1					* stars y spin velocity
spin_z
	ds.w	1					* stars z spin velocity

delta_y
	ds.w	1					* stars velocity

theta_x
	ds.w	1					* stars rotation x angle
theta_y
	ds.w	1					* stars rotation y angle
theta_z
	ds.w	1					* stars rotation z angle

sin_x
	ds.w	1					* SIN theta_x
cos_x
	ds.w	1					* COS theta_x
sin_y
	ds.w	1					* SIN theta_y
cos_y
	ds.w	1					* COS theta_y
sin_z
	ds.w	1					* SIN theta_z
cos_z
	ds.w	1					* COS theta_z

* stars buffer

x_coords
	ds.w	star_count				* stars x co-ordinates
y_coords
	ds.w	star_count				* stars y co-ordinates
z_coords
	ds.w	star_count				* stars z co-ordinates

* translated stars buffer

x_buffer
	ds.w	star_count				* star x co-ordinates
y_buffer
	ds.w	star_count				* star y co-ordinates
z_buffer
	ds.w	star_count				* star z co-ordinates

	END	start


*************************************************************************************
